<?php
/**
 * Register custom settings for Gap Block
 *
 * @package gap-block
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

add_action( 'wdp_blocks_summary_settings', array( 'WDPGB_Settings', 'setting_form' ) );
add_action( 'admin_enqueue_scripts', array( 'WDPGB_Settings', 'enqueue_scripts' ) );
add_action( 'rest_api_init', array( 'WDPGB_Settings', 'register_endpoint' ) );

add_filter( 'wdp_blocks_summary_has_settings', '__return_true' );

/**
 * Settings class
 */
abstract class WDPGB_Settings {

	/**
	 * Breakpoints option name
	 *
	 * @var string
	 */
	public static $option_name = 'wdpgb-responsive-breakpoints';

	/**
	 * Register endpoint
	 */
	public static function register_endpoint() {

		register_rest_route(
			'wdpgb/v1',
			'/update-breakpoints',
			array(
				'methods'             => 'POST',
				'callback'            => array( get_class(), 'update_brakpoints' ),
				'args'                => array(
					'nonce' => array(
						'required'          => true,
						'validate_callback' => function( $nonce ) {
							return wp_verify_nonce( $nonce, 'wdpgb_update_breakpoints' );
						},
					),
				),
				'permission_callback' => function() {
					return current_user_can( 'manage_options' );
				},
			)
		);
	}

	/**
	 * Get setting form content
	 */
	public static function setting_form() {

		?>
		<div id="wdp-blocks-summary__setting__responsive-breakpoints" class="wdp-blocks-summary__setting">
			<p class="wdp-blocks-summary__setting__loading-notice"><?php echo esc_html( __( 'Loading...', 'gap-block' ) ); ?></p>
		</div>
		<?php
	}

	/**
	 * Enqueue settings JS script
	 */
	public static function enqueue_scripts() {

		$current_screen = get_current_screen();

		if ( 'settings_page_wdp-blocks-summary' === $current_screen->id ) {

			wp_enqueue_style( 'wdp-gap-block-settings', plugins_url( 'build/settings.css', WDPGB_MAIN_FILE ), array(), WDPGB_VERSION );
			wp_enqueue_script( 'wdp-gap-block-settings', plugins_url( 'build/settings.min.js', WDPGB_MAIN_FILE ), array( 'wp-element', 'wp-i18n' ), WDPGB_VERSION, true );

			wp_localize_script(
				'wdp-gap-block-settings',
				'wdpgbSettings',
				array(
					'breakpoints'        => self::get_breakpoints(),
					'defaultBreakpoints' => self::get_default_breakpoints(),
					'nonces'             => array(
						'updateBreakpoints' => wp_create_nonce( 'wdpgb_update_breakpoints' ),
						'wpRest'            => wp_create_nonce( 'wp_rest' ),
					),
				)
			);
		}
	}

	/**
	 * Get default breakpoints
	 */
	public static function get_default_breakpoints() {

		$defaults = array(
			'lg' => (int) apply_filters( 'wdpgb_lg_breakpoint', 1569 ),
			'md' => (int) apply_filters( 'wdpgb_md_breakpoint', 1199 ),
			'sm' => (int) apply_filters( 'wdpgb_sm_breakpoint', 991 ),
			'xs' => (int) apply_filters( 'wdpgb_xs_breakpoint', 781 ),
		);

		if ( ! self::are_breakpoints_valid( $defaults ) ) {
			$defaults = array(
				'lg' => 1569,
				'md' => 1199,
				'sm' => 991,
				'xs' => 781,
			);
		}

		return $defaults;
	}

	/**
	 * Get responsive breakpoints
	 *
	 * @throws Exception If breakpoints are invalid.
	 */
	public static function get_breakpoints() {

		$breakpoints = get_option( self::$option_name );
		$defaults    = self::get_default_breakpoints();

		if ( false !== $breakpoints ) {
			try {
				$breakpoints = json_decode( $breakpoints, true );
				if ( ! self::are_breakpoints_valid( $breakpoints ) ) {
					throw new Exception( 'invalid_breakpoints' );
				}
				return $breakpoints;
			} catch ( Exception $exception ) {
				return $defaults;
			}
		}

		return $defaults;
	}

	/**
	 * Check if breakpoints are valid
	 *
	 * @param array $breakpoints Breakpoints.
	 */
	private static function are_breakpoints_valid( $breakpoints ) {

		if ( 4 !== count( $breakpoints ) ) {
			return false;
		}

		foreach ( array( 'xs', 'sm', 'md', 'lg' ) as $breakpoint ) {
			if ( ! isset( $breakpoints[ $breakpoint ] ) || ! is_int( $breakpoints[ $breakpoint ] ) ) {
				return false;
			}
		}

		foreach ( $breakpoints as $breakpoint ) {
			if ( 1 > $breakpoint || 9998 < $breakpoint ) {
				return false;
			}
		}

		return $breakpoints['lg'] > $breakpoints['md'] && $breakpoints['md'] > $breakpoints['sm'] && $breakpoints['sm'] > $breakpoints['xs'];
	}

	/**
	 * Handle REST request
	 * - update breakpoints
	 *
	 * @param object $request Request object.
	 */
	public static function update_brakpoints( $request ) {

		$body = json_decode( $request->get_body(), true );

		if ( ! isset( $body['breakpoints'] ) || ! self::are_breakpoints_valid( $body['breakpoints'] ) ) {
			return new WP_REST_Response(
				array(
					'code'    => 'invalid_breakpoints',
					'message' => __( 'Breakpoints data invalid.', 'gap-block' ),
					'data'    => array(
						'status' => 400,
					),
				),
				400
			);
		}

		if ( true === update_option( self::$option_name, wp_json_encode( $body['breakpoints'] ) ) ) {
			return new WP_REST_Response( array( 'status' => 'ok' ), 200 );
		}

		return new WP_REST_Response( array( 'status' => 'error' ), 500 );
	}
}
